#include <iostream>
#include <vector>
#include <string>
#include <sstream>
#include <cmath>
#include <cstdlib>
#include <cstring>
#include <cstdio>
#include <cassert>
#include <algorithm>
#include <list>
#include <queue>
#include <stack>
#include <functional>
#include <set>
#include <map>

using namespace std;

typedef long double LD;
typedef long long LL;
typedef pair<int, int> pii;

#define X first
#define Y second
#define dbg(x) { cerr << #x << " = " << x << endl; }
#define iss istringstream
#define oss ostringstream
#define pb push_back
#define mp make_pair
const string NAME = "GGv5";
const int inf = 1000000000;
const int MAX_IT = 10;
const int MAX_PATH = 50;
const int MAX_TRY = 10;

struct point_t
{
	int x, y;
	point_t() {}
	point_t(int x, int y) : x(x), y(y) {}
	pii to_pair() const { return mp(x, y); }
	friend bool operator < (const point_t &a, const point_t &b)
	{
		return a.to_pair() < b.to_pair();
	}

	friend point_t operator - (const point_t &a, const point_t &b)
	{
		return point_t(a.x - b.x, a.y - b.y);
	}
	friend point_t operator + (const point_t &a, const point_t &b)
	{
		return point_t(a.x + b.x, a.y + b.y);
	}
};

const int NDIR = 8;
point_t dir[NDIR];
int posd[3][3];

void init()
{
	int cdir = 0;
	for(int dx = -1; dx <= 1; ++dx)
		for(int dy = -1; dy <= 1; ++dy)
			if(dx || dy)
			{
				posd[dx + 1][dy + 1] = cdir;
				dir[cdir++] = point_t(dx, dy);
			}
}

int get_dir(int dx, int dy) { return posd[dx + 1][dy + 1]; }
int get_dir(const point_t &a) { return posd[a.x + 1][a.y + 1]; }

const int MAXN = 15;
const int MAXM = 21;
const int MAXX = MAXN - 1;
const int MAXY = MAXM - 1;

struct field_t
{
	bool used[MAXN][MAXM];
	bool used_edges[MAXN][MAXM][8];
	point_t last;

	field_t() 
	{
		memset(used, 0, sizeof used);
		memset(used_edges, 0, sizeof used_edges);
	}

	field_t(const field_t &other)
	{
		memcpy(used, other.used, sizeof used);
		memcpy(used_edges, other.used_edges, sizeof used_edges);
	}

	bool is_used(const point_t &a) const
	{ return used[a.x][a.y]; }
	bool is_used(int x, int y) const
	{ return used[x][y]; }

	bool is_used(const point_t &a, const point_t &b) const
	{ return used_edges[a.x][a.y][get_dir(b - a)]; }
	
	void mark_point(const point_t &a)
	{ used[a.x][a.y] = true; }
	void mark_point(int x, int y)
	{ used[x][y] = true; }

	void mark_edge(const point_t &a, const point_t &b, bool mark_pt = true)
	{
		if(mark_pt)
		{
			mark_point(a);
			mark_point(b);
		}
		used_edges[a.x][a.y][get_dir(b - a)] = true;
		used_edges[b.x][b.y][get_dir(a - b)] = true;
	}
	void unmark_edge(const point_t &a, const point_t &b)
	{
		
		used_edges[a.x][a.y][get_dir(b - a)] = false;
		used_edges[b.x][b.y][get_dir(a - b)] = false;
	}

	void unmark_point(const point_t &a)
	{ used[a.x][a.y] = false; }
	void unmark_point(int x, int y)
	{ used[x][y] = false; }

	
	
	void init()
	{
		memset(used, 0, sizeof used);
		memset(used_edges, 0, sizeof used_edges);
		
		last = point_t(7, 10);
		mark_point(last);
		for(int y = 0; y + 1 <= MAXY; ++y)
		{
			mark_edge(point_t(0, y), point_t(0, y + 1));
			mark_edge(point_t(MAXX, y), point_t(MAXX, y + 1));
		}

		for(int x = 0; x + 1 <= MAXX; ++x)
		{
			mark_edge(point_t(x, 0), point_t(x + 1, 0));
			mark_edge(point_t(x, MAXY), point_t(x + 1, MAXY));
		}
		unmark_point(point_t(8, 0));
		unmark_point(point_t(6, 0));
		unmark_point(point_t(7, 0));

		unmark_point(point_t(8, MAXY));
		unmark_point(point_t(6, MAXY));
		unmark_point(point_t(7, MAXY));

	}

	void mark_path(const vector<point_t> &path)
	{
		for(int i = 0; i + 1 < path.size(); ++i)
			mark_edge(path[i], path[i + 1]);
		last = path.back();
	}

	vector<point_t> get_random_path() 
	{
		memset(was, 0, sizeof was);
		vector<point_t> res;
		dfs(last, res);
		for(int i = 0; i + 1 < res.size(); ++i)
			unmark_edge(res[i], res[i + 1]);
		return res;
	}

	bool check(const point_t &a) const
	{
		if(a.x < 0 || a.y < 0 || a.x > MAXX || a.y > MAXY)
			return false;
		return true;
	}

	vector<point_t> get_neigh(const point_t &u) const
	{
		vector<point_t> neigh;
		for(int i = 0; i < NDIR; ++i)
		{
			point_t v = u + dir[i];
			if(check(v))
				neigh.pb(v);
		}
		return neigh;
	}

private:
	bool was[MAXN][MAXM];
	bool dfs(const point_t &u, vector<point_t> &path)
	{
		path.pb(u);
		if(!is_used(u))
			return true;
		was[u.x][u.y] = true;
		vector<point_t> neigh = get_neigh(u);
		random_shuffle(neigh.begin(), neigh.end());
		for(int i = 0; i < neigh.size(); ++i)
		{
			point_t v = neigh[i];
			if(!is_used(u, v) && !was[v.x][v.y])
			{
				mark_edge(u, v, false);
				if(dfs(v, path))
					return true;
				unmark_edge(u, v);
			}
		}
		path.pop_back();
		return false;
	}
};
struct game_t;

int simulate(game_t &game);


struct game_t
{
	bool is_first;
	field_t field;
	
	game_t() { is_first = false; }
	game_t(const game_t &other)	{ is_first = other.is_first; field = other.field; }
	string get_name() const { return NAME; }
	void init_field() { field.init(); }
	void add_path(const vector<point_t> &path) { field.mark_path(path); }

	vector<point_t> get_path() 
	{
		vector<point_t> bpath;
		int bscore = -inf;
		
		for(int i = 0; i < MAX_PATH; ++i)
		{
			
			vector<point_t> path = field.get_random_path();
			int cscore = 0;
			for(int j = 0; j < MAX_TRY; ++j)
			{
				game_t cur = *this;
				cur.add_path(path);
				int res = simulate(cur);
				cscore += res;
			}
			if(bscore < cscore)
			{
				bscore = cscore;
				bpath = path;
			}
		}
		return bpath;//vector<point_t>();
	}
};

int simulate(game_t &game)
{
	int cur = 1;

	for(int it = 0; it < MAX_IT; ++it)
	{
		if(game.field.last.x == MAXX)
		{
			if(game.is_first)
				return 30;
			else
				return -30;
		}
		if(game.field.last.x == 0)
		{
			if(!game.is_first)
				return 30;
			else
				return -30;
		}
		vector<point_t> path = game.field.get_random_path();
		if(path.size() == 0)
			return cur * 30;

		game.add_path(path);
		cur = -cur;
	}
	int my_gate = 0;
	if(!game.is_first)
		my_gate = MAXY;

	int dst = my_gate - game.field.last.y;
	if(dst < 0)
		dst = -dst;
	if(dst < 10)
		return -dst-10;
	else 
		return 10+(dst - 10) / 2;

	 

}

void run()
{
	game_t game;
	game.init_field();

	string cmd;
	while(getline(cin, cmd))
	{
		if(cmd == "Quit")
			return;
		if(cmd == "Name")
		{
			cout << game.get_name() << endl;
		} else
		{
			if(cmd == "Start")
			{
				game.is_first = true;
			} else 
			{
				iss is(cmd);
				int n;
				is >> n;
				vector<point_t> path(n);
				for(int i = 0; i < n; ++i)
					is >> path[i].x >> path[i].y;

				game.add_path(path);
			}

			vector<point_t> path = game.get_path();
			game.add_path(path);
			oss os;
			os << path.size();
			for(int i = 0; i < path.size(); ++i)
				os << " " << path[i].x << " " << path[i].y;
			cout << os.str() << endl;
		}
		cout.flush();
	}
	
}

int main()
{
	init();
	run();

	return 0;
}
