using System;
using System.Collections.Generic;

namespace CSharpBot
{
    internal class Utils
    {
        public static List<int> Split(string str)
        {
            var args = str.Split(' ');
            var res = new List<int>();
            for (int i = 0; i < args.Length; i++)
            {
                if (!string.IsNullOrEmpty(args[i]))
                    res.Add(int.Parse(args[i]));
            }
            return res;
        }
    }

    internal class Point
    {
        public int x, y;

        public Point(int x, int y)
        {
            this.x = x;
            this.y = y;
        }

        public bool Equals(Point other)
        {
            if (ReferenceEquals(null, other)) return false;
            if (ReferenceEquals(this, other)) return true;
            return other.x == x && other.y == y;
        }

        public override bool Equals(object obj)
        {
            if (ReferenceEquals(null, obj)) return false;
            if (ReferenceEquals(this, obj)) return true;
            if (obj.GetType() != typeof(Point)) return false;
            return Equals((Point)obj);
        }

        public override int GetHashCode()
        {
            unchecked
            {
                return (x * 397) ^ y;
            }
        }
    }

    public class Bot
    {
        private readonly int[,] d = new[,] { { -1, -1 }, { -1, 0 }, { -1, 1 }, { 0, -1 }, { 0, 1 }, { 1, -1 }, { 1, 0 }, { 1, 1 } };
        private const int columns = 20;
        private const int goalLength = 4;
        private const int lines = 14;
        private const int goalStart = ((lines - goalLength) / 2);
        private const int goalFinish = goalStart + goalLength;
        private readonly bool[, , ,] moves;
        private bool[,] mark;
        private int ourColumn;
        private Point[,] parent;
        private int resDistToCenter;
        private int resDistToMyGoal;
        private int resx, resy;
        private int x, y;

        public Bot()
        {
            moves = new bool[lines + 1, columns + 1, lines + 1, columns + 1];
            for (int i = 0; i <= lines; i++)
                for (int j = 0; j <= columns; j++)
                    for (int k = 0; k <= lines; k++)
                        for (int l = 0; l <= columns; l++)
                            moves[i, j, k, l] = false;
        }

        private int MovesCount(int x, int y)
        {
            int res = 0;
            for (int i = 0; i <= lines; i++)
                for (int j = 0; j <= columns; j++)
                    if (moves[x, y, i, j])
                        res++;
            return res;
        }

        private void Fill(List<Point> v)
        {
            if (v.Count == 0)
                return;
            for (int i = 1; i < v.Count; i++)
            {
                moves[v[i].x, v[i].y, v[i - 1].x, v[i - 1].y] = true;
                moves[v[i - 1].x, v[i - 1].y, v[i].x, v[i].y] = true;
            }
            x = v[v.Count - 1].x;
            y = v[v.Count - 1].y;
        }

        private void MakeEnemyMove(String str)
        {
            var nums = Utils.Split(str);
            int moveLen = nums[0];
            var points = new List<Point>();
            for (int i = 0; i < moveLen; i++)
            {
                int x = nums[i * 2 + 1];
                int y = nums[i * 2 + 2];
                points.Add(new Point(x, y));
            }
            Fill(points);
        }

        private void InitDfs()
        {
            mark = new bool[lines + 1, columns + 1];
            for (int i = 0; i <= lines; i++)
                for (int j = 0; j <= columns; j++)
                    mark[i, j] = false;

            parent = new Point[lines + 1, columns + 1];
            resx = -1;
            resy = -1;
            resDistToMyGoal = columns + 2;
            resDistToCenter = lines + 2;
        }

        private void Dfs(int curx, int cury, int lastx, int lasty)
        {
            mark[curx, cury] = true;
            parent[curx, cury] = new Point(lastx, lasty);
            for (int dir = 0; dir < 8; dir++)
            {
                int dx = d[dir, 0];
                int dy = d[dir, 1];
                int x = curx + dx;
                int y = cury + dy;
                if (x < 0 || x > lines || y < 0 || y > columns) continue;
                if (!moves[curx, cury, x, y] && !mark[x, y])
                {
                    if (ourColumn == y && (goalStart < x && goalFinish > x))
                    {
                        parent[x, y] = new Point(curx, cury);
                        mark[x, y] = true;
                        resx = x;
                        resy = y;
                        return;
                    }
                    else
                    {
                        if (MovesCount(x, y) == 0)
                        {
                            parent[x, y] = new Point(curx, cury);
                            mark[x, y] = true;
                            int distToMyGoal = Math.Abs(y - ourColumn);
                            if (distToMyGoal == 0) distToMyGoal = columns + 1;
                            int distToCenter = Math.Abs(x - lines / 2);
                            if (distToMyGoal < resDistToMyGoal ||
                                (distToMyGoal == resDistToMyGoal && distToCenter < resDistToCenter))
                            {
                                resx = x;
                                resy = y;
                                resDistToMyGoal = distToMyGoal;
                                resDistToCenter = distToCenter;
                            }
                        }
                        else
                        {
                            Dfs(x, y, curx, cury);
                        }
                    }
                }
            }
        }

        private List<Point> CollectMove(int x, int y)
        {
            var res = new List<Point>();
            while (!(x == -1 && y == -1))
            {
                var p = new Point(x, y);
                res.Add(p);
                x = parent[p.x, p.y].x;
                y = parent[p.x, p.y].y;
            }
            var resArray = res.ToArray();
            Array.Reverse(resArray);
            res = new List<Point>(resArray);
            return res;
        }

        private void MakeMove()
        {
            InitDfs();
            Dfs(x, y, -1, -1);
            var myMove = CollectMove(resx, resy);
            Console.Write(myMove.Count);
            for (int i = 0; i < myMove.Count; i++)
                Console.Write(" " + myMove[i].x + " " + myMove[i].y);
            Console.WriteLine();
            Fill(myMove);
        }

        private void FillEdge(int x1, int y1, int x2, int y2)
        {
            if (x1 < 0 || x1 > lines) return;
            if (x2 < 0 || x2 > lines) return;
            if (y1 < 0 || y1 > columns) return;
            if (y2 < 0 || y2 > columns) return;
            moves[x1, y1, x2, y2] = true;
        }

        private void InitField(bool enemyFirst)
        {
            x = lines / 2;
            y = columns / 2;
            ourColumn = enemyFirst ? columns : 0;

            for (int i = 0; i <= lines; i++)
            {
                if (i <= goalStart || i >= goalFinish)
                {
                    FillEdge(i, 0, i - 1, 0);
                    FillEdge(i, columns, i - 1, columns);
                    FillEdge(i, 0, i + 1, 0);
                    FillEdge(i, columns, i + 1, columns);
                }
            }
            for (int j = 0; j <= columns; j++)
            {
                FillEdge(0, j, 0, j - 1);
                FillEdge(0, j, 0, j + 1);
                FillEdge(lines, j, lines, j - 1);
                FillEdge(lines, j, lines, j + 1);
            }
        }

        private void Run()
        {
            bool firstMove = true;
            //var reader = new System.IO.StreamReader("input.txt");
            while (true)
            {
                String str = Console.ReadLine();
                //reader.ReadLine();
                //String str = reader.ReadLine();
                if (str == "Quit")
                    return;
                if (str == "Name")
                {
                    Console.WriteLine("POVT");
                    Console.Out.Flush();
                    continue;
                }
                if (str != "Start")
                {
                    if (firstMove)
                    {
                        InitField(false);
                        firstMove = false;
                    }
                    MakeEnemyMove(str);
                }
                if (firstMove)
                {
                    InitField(true);
                    firstMove = false;
                }
                MakeMove();
                Console.Out.Flush();
            }
        }

        public static void Main(String[] args)
        {
            new Bot().Run();
        }
    }
}