import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Scanner;

class Utils {
    public static List<Integer> Split(String str) {
        String[] args = str.split(" ");
        List<Integer> res = new ArrayList<Integer>();
        for (int i = 0; i < args.length; i++) {
            if (!args[i].equals(""))
                res.add(Integer.parseInt(args[i]));
        }
        return res;
    }
}

class Point {
    public int x, y;

    public Point(int x, int y) {
        this.x = x;
        this.y = y;
    }

    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;

        Point point = (Point) o;

        if (x != point.x) return false;
        if (y != point.y) return false;

        return true;
    }
}

public class Bot {
    private final int lines = 14;
    private final int columns = 20;
    private final int goalLength = 4;
    private final int goalStart = ((lines - goalLength) / 2);
    private final int goalFinish = goalStart + goalLength;

    int ourColumn;
    Boolean[][][][] moves;
    int x, y;


    int MovesCount(int x, int y) {
        int res = 0;
        for (int i = 0; i <= lines; i++)
            for (int j = 0; j <= columns; j++)
                if (moves[x][y][i][j])
                    res++;
        return res;
    }

    public Bot() {
        moves = new Boolean[lines + 1][columns + 1][lines + 1][columns + 1];
        for (int i = 0; i <= lines; i++)
            for (int j = 0; j <= columns; j++)
                for (int k = 0; k <= lines; k++)
                    for (int l = 0; l <= columns; l++)
                        moves[i][j][k][l] = false;
    }

    void Fill(List<Point> v) {
        for (int i = 1; i < v.size(); i++) {
            moves[v.get(i).x][v.get(i).y][v.get(i - 1).x][v.get(i - 1).y] = true;
            moves[v.get(i - 1).x][v.get(i - 1).y][v.get(i).x][v.get(i).y] = true;
        }
        x = v.get(v.size() - 1).x;
        y = v.get(v.size() - 1).y;
    }

    void MakeEnemyMove(String str) {
        List<Integer> nums = Utils.Split(str);
        int moveLen = nums.get(0);
        List<Point> points = new ArrayList<Point>();
        for (int i = 0; i < moveLen; i++) {
            int x = nums.get(i * 2 + 1);
            int y = nums.get(i * 2 + 2);
            points.add(new Point(x, y));
        }
        Fill(points);
    }

    Boolean[][] mark;
    Point[][] parent;
    int resx, resy;
    int resDistToMyGoal;
    int resDistToCenter;
    
    boolean checkPoint(int distToGoal1, int distToCenter1, int distToGoal2, int distToCenter2){
    	if (distToGoal1 > distToGoal2){
    		return true;
    	} else if (distToGoal1 < distToGoal2){
    		return false;
    	} else if (distToGoal1 == distToGoal2){
    		if (distToGoal1 % 2 == 0){
    			if (distToCenter1 < distToCenter2){
    				return true;
    			} else {
    				return false;
    			}
    		} else {
    			if (distToCenter1 > distToCenter2){
    				return true;
    			} else {
    				return false;
    			}
    		}
    	}
    	return true;
    }

    void InitDfs() {
        mark = new Boolean[lines + 1][columns + 1];
        for (int i = 0; i <= lines; i++)
            for (int j = 0; j <= columns; j++)
                mark[i][j] = false;

        parent = new Point[lines + 1][columns + 1];
        resx = -1;
        resy = -1;
//        resDistToMyGoal = columns + 2;
        resDistToCenter = lines + 2;
        resDistToMyGoal = -1;
    }

    int[][] d = {{-1, -1}, {-1, 0}, {-1, 1}, {0, -1}, {0, 1}, {1, -1}, {1, 0}, {1, 1}};

    void Dfs(int curx, int cury, int lastx, int lasty) {
        mark[curx][cury] = true;
        parent[curx][cury] = new Point(lastx, lasty);
        for (int dir = 0; dir < 8; dir++) {
            int dx = d[dir][0];
            int dy = d[dir][1];
            int x = curx + dx;
            int y = cury + dy;
            if (x < 0 || x > lines || y < 0 || y > columns) continue;
            if (!moves[curx][cury][x][y] && !mark[x][y]) {
                if (MovesCount(x, y) == 0) {
                    parent[x][y] = new Point(curx, cury);
                    mark[x][y] = true;
                    int distToMyGoal = Math.abs(y - ourColumn);
//                    if (distToMyGoal == 0) distToMyGoal = columns + 1;
                    int distToCenter = Math.abs(x - lines / 2);
                    if (checkPoint(distToMyGoal, distToCenter, resDistToMyGoal, resDistToCenter)) {
                        resx = x;
                        resy = y;
                        resDistToMyGoal = distToMyGoal;
                        resDistToCenter = distToCenter;
                    }
                } else {
                    Dfs(x, y, curx, cury);
                }
            }
        }
    }

    List<Point> CollectMove(int x, int y) {
        List<Point> res = new ArrayList<Point>();
        while (!(x == -1 && y == -1)) {
            Point p = new Point(x, y);
            res.add(p);
            x = parent[p.x][p.y].x;
            y = parent[p.x][p.y].y;
        }
        Collections.reverse(res);
        return res;
    }

    void MakeMove() {
        InitDfs();
        Dfs(x, y, -1, -1);
        List<Point> myMove = CollectMove(resx, resy);
        System.out.print(myMove.size());
        for (int i = 0; i < myMove.size(); i++)
            System.out.print(" " + myMove.get(i).x + " " + myMove.get(i).y);
//        System.out.println(" Zhopa " + resDistToMyGoal + " " + columns);
        System.out.println();
        Fill(myMove);
    }

    void FillEdge(int x1, int y1, int x2, int y2) {
        if (x1 < 0 || x1 > lines) return;
        if (x2 < 0 || x2 > lines) return;
        if (y1 < 0 || y1 > columns) return;
        if (y2 < 0 || y2 > columns) return;
        moves[x1][y1][x2][y2] = true;
    }

    void InitField(Boolean enemyFirst) {
        x = lines / 2;
        y = columns / 2;
        ourColumn = enemyFirst ? columns : 0;

        for (int i = 0; i <= lines; i++) {
            if (i <= goalStart || i >= goalFinish) {
                FillEdge(i, 0, i - 1, 0);
                FillEdge(i, columns, i - 1, columns);
                FillEdge(i, 0, i + 1, 0);
                FillEdge(i, columns, i + 1, columns);
            }
        }
        for (int j = 0; j <= columns; j++) {
            FillEdge(0, j, 0, j - 1);
            FillEdge(0, j, 0, j + 1);
            FillEdge(lines, j, lines, j - 1);
            FillEdge(lines, j, lines, j + 1);
        }
    }

    void Run() {
        Boolean firstMove = true;
        Scanner scanner = new Scanner(System.in);
        while (true) {
            String str = scanner.nextLine();
            if (str.equals("Quit"))
                return;
            if (str.equals("Name")) {
                System.out.println("Lenyuk");
                System.out.flush();
                continue;
            }
            if (!str.equals("Start")) {
                if (firstMove) {
                    InitField(true);
                    firstMove = false;
                }
                MakeEnemyMove(str);
            }
            if (firstMove) {
                InitField(false);
                firstMove = false;
            }
            MakeMove();
            System.out.flush();
        }
    }

    public static void main(String[] args) {
        new Bot().Run();
    }
}