#include <vector>
#include <string>
#include <cstdio>
#include <algorithm>
#include <queue>
#include <sstream>
#include <ctime>
#include <map>
#include <set>

using namespace std;
#define FOR(i, n) for (int i = 0; i < (int)(n); i++)

void CHK(bool b) { if (!b) throw 42; }
const int MAX_TIME = 100 * 10000;
const int DEPTHO = 1;

template<class T> T Abs(T x) { return x < 0 ? -x : x; }

struct P{
	typedef const P& CP;
	short x, y;
	P() {}
	P(short x, short y) : x(x), y(y) { }
	P(int id) : x(id / 200), y(id % 200) { }
	int Id() const { return x * 200 + y; }

	friend P operator + (const P& a, const P& b) { return P(a.x + b.x, a.y + b.y); }
	friend P operator - (const P& a, const P& b) { return P(a.x - b.x, a.y - b.y); }
	friend bool operator == (const P& a, const P& b) { return a.x == b.x && a.y == b.y; }
	friend bool operator < (const P& a, const P& b) { return a.x < b.x || a.x == b.x && a.y < b.y; }

};
typedef const P& CP;
typedef vector<P> VP;

static const P dp[8] = { P(-1, 0), P(1, 0), P(0, -1), P(0, 1), P(-1, -1), P(1, 1), P(-1, 1), P(1, -1)  };

int findDir(CP d){	
	FOR(i, 8){
		if (dp[i] == d)
			return i;
	}
	return -1;
}

const int SZ_X = 20;
const int SZ_Y = 14;
const P SZ = P(SZ_X, SZ_Y);
const P BALL = P(10, 7);
const P GOAL[2][2] = {{P(0, 5), P(0, 9)}, {P(20, 5), P(20, 9)}};
const double ZAEBIS = 1e9;

bool inside(const P& p){
	return p.x >= 0 && p.x <= SZ.x && p.y >= 0 && p.y <= SZ.y;
}

void parseMove(const string& str, VP& pp){
	stringstream ss(str);
	int n;
	ss >> n;
	pp.resize(n);
	FOR(i, n){
		ss >> pp[i].y >> pp[i].x;
	}
}

struct Game{
	char mm[SZ_X + 1][SZ_Y + 1][8];
	char was[SZ_X + 1][SZ_Y + 1];
	P ball;
	int team;
	int start_time;

	bool isGate(CP p, int team){		
		CP a = GOAL[team][0];
		CP b = GOAL[team][1];
		return p.x == a.x && a.y < p.y && p.y < b.y;
	}
	bool isBorder(CP p){
		if (isGate(p, 0) || isGate(p, 1))
			return 0;
		return p.x == 0 || p.y == 0 || p.x == SZ_X || p.y == SZ_Y;
	}	


	void init(bool weAreLeft){
		memset(pr, -1, sizeof(pr));
		team = weAreLeft ? 0 : 1;
		memset(mm, 0, sizeof(mm));
		memset(was, 0, sizeof(was));
		ball = BALL;
		for (int x = 0; x < SZ_X; x++){
			setEdge(P(x, 0), P(x + 1, 0), 1);
			setEdge(P(x, SZ_Y), P(x + 1, SZ_Y), 1);

		}
		for (int y = 0; y < SZ_Y; y++){
			setEdge(P(0, y), P(0, y + 1), 1);
			setEdge(P(SZ_X, y), P(SZ_X, y + 1), 1);
		}
		FOR(x, SZ_X + 1){
			was[x][0] = 1;
			was[x][SZ_Y] = 1;
		}
		FOR(y, SZ_Y + 1){
			if (!(5 < y && y < 9)){
				was[0][y] = 1;
				was[SZ_X][y] = 1;
			}
		}

		was[ball.x][ball.y] = 1;
	}
	void setEdge(CP a, CP b, bool v){
		int dir = findDir(b - a);
		mm[a.x][a.y][dir] = v;
		mm[b.x][b.y][dir ^ 1] = v;
	}
	bool getEdge(CP a, CP b){
		int dir = findDir(b - a);
		CHK(mm[a.x][a.y][dir] == mm[b.x][b.y][dir ^ 1]);
		return mm[a.x][a.y][dir];		
	}
	void applyMove(const VP& move){
		for (int i = 0; i < move.size(); i++){
			CP p = move[i];
			CHK((i + 1 == move.size()) ? (!was[p.x][p.y]) : was[p.x][p.y]);
			was[p.x][p.y] = 1;
			if (i > 0){
				setEdge(move[i - 1], move[i], true);				
			}
		}
		ball = move.back();
	}	
	void backupMove(const VP& move){
		was[ball.x][ball.y] = 0;
		for (int i = move.size() - 1; i > 0; i--){			
			setEdge(move[i - 1], move[i], false);
		}
		ball = move.front();
	}
	typedef double Score;	
	int pr[SZ_X + 1][SZ_Y + 1];
	void bfs(CP start, VP& dests){
		static int Q[1 << 20];
		int qa = 0, qb = 0;
		Q[qb++] = start.Id();
		int D[SZ_X + 1][SZ_Y + 1];
		memset(pr, -1, sizeof(pr));
		memset(D, -1, sizeof(D));

		D[start.x][start.y] = 0;
		while (qa != qb){
			P u(Q[qa++]);
			FOR(i, 8){
				P v = u + dp[i];
				if (!inside(v))
					continue;
				if (mm[u.x][u.y][i])
					continue;
				if (D[v.x][v.y] != -1)
					continue;
				D[v.x][v.y] = D[u.x][u.y] + 1;
				pr[v.x][v.y] = u.Id();
				if (was[v.x][v.y]){
					Q[qb++] = v.Id();
				}
				else {
					if (!isBorder(v))
						dests.push_back(v);
				}
			}
		}
	}
	void bfs2(CP start, VP& dests){
		static int Q[1 << 20];
		int qa = 0, qb = 0;
		Q[qb++] = start.Id();
		int D[SZ_X + 1][SZ_Y + 1];

		memset(D, -1, sizeof(D));

		D[start.x][start.y] = 0;
		while (qa != qb){
			P u(Q[qa++]);
			FOR(i, 8){
				P v = u + dp[i];
				if (!inside(v))
					continue;
				if (mm[u.x][u.y][i])
					continue;
				if (D[v.x][v.y] != -1)
					continue;
				D[v.x][v.y] = D[u.x][u.y] + 1;
				if (was[v.x][v.y]){
					Q[qb++] = v.Id();
				}
				else {
					if (!isBorder(v))
						dests.push_back(v);
				}
			}
		}
	}

	void findMove(VP& move){
		if (ball.x == 19 && ball.y == 8){
			ball = ball;
		}
		start_time = clock();
		goDeep(team, DEPTHO, true);
		move.clear();
		FOR(i, bestPath.size()){
			move.push_back(P(bestPath[i]));
		}		
	}
	void findPath(P dest, VP& path){
		path.clear();
		for (;;){
			path.push_back(dest);
			int id = pr[dest.x][dest.y];
			if (id == -1)
				break;
			dest = P(id);
		}
		reverse(path.begin(), path.end());
	}
	int dist(CP a, CP b){
		return max(Abs(b.x - a.x), Abs(a.y - b.y));
	}
	int distToGoal(int team, CP p){
		int x = GOAL[team][0].x;
		int res = SZ_X * 2;
		for (int i = 6; i <= 8; i++)
			res = min(res, dist(P(x, i), p));
		return res;
	}
	Score currScore(int team){
		//int toEnemy = distToGoal(team ^ 1, ball);
		int toOur = distToGoal(team, ball);
		VP dests;
		bfs2(ball, dests);
		int res = -1;
		FOR(i, dests.size()){
			int nr = distToGoal(team ^ 1, dests[i]);
			if (res == -1 || nr < res)
				res = nr;
		}
		return 2 * res - toOur;
	}
	bool isTimeOut(){
		return clock() - start_time > MAX_TIME;
	}
	Score bestScore;
	VP bestPath;
	Score goDeep(int team, int depth, bool save = 0){
		if (isGate(ball, team))
			return -ZAEBIS;
		if (isGate(ball, team ^ 1))
			return +ZAEBIS;
		if (depth == 0 || isTimeOut())
			return currScore(team);

		Score res = -1e12;
		VP dests;
		bfs(ball, dests);
		FOR(i, dests.size()){
			CP dest = dests[i];

			VP path;
			findPath(dest, path);			
			applyMove(path);
			double nr = -goDeep(team ^ 1, depth - 1);
			if (nr > res){
				res = nr;
				if (save)
					bestPath = path;
			}
			backupMove(path);
		}
		return res;
	}	
};


int main()
{
	//freopen("inp.txt", "r", stdin);
	bool firstMove = true;	
	static char buf[1 << 20];

	Game game;
	for (;;) {
		gets(buf);
		/*
		FILE* du = fopen("dump.txt", "a");
		fprintf(du, "%s\n", buf);
		fclose(du);
		*/
		string str = string(buf);

		if (str == "Quit")
			break;
		else if (str == "Name") {
			printf("/|EXA\n");
		}
		else {
			if (str != "Start") {
				if (firstMove) {
					game.init(false);
					firstMove = false;
				}
				VP enemyMove;
				parseMove(str, enemyMove);
				game.applyMove(enemyMove);				
			}
			if (firstMove) {
				game.init(true);
				firstMove = false;
			}			
			VP ourMove;
			game.findMove(ourMove);
			game.applyMove(ourMove);
			printf("%d", ourMove.size());
			FOR(i, ourMove.size()){
				printf(" %d %d", ourMove[i].y, ourMove[i].x);
			}
			printf("\n");
		}		
		fflush(stdout);
	}	
	return 0;
}